"""Constants for geometry submodels."""

#  This file is part of FAST-OAD_CS25
#  Copyright (C) 2022 ONERA & ISAE-SUPAERO
#  FAST is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.

SERVICE_FUSELAGE_GEOMETRY_BASIC = "service.geometry.fuselage.basic"
SERVICE_FUSELAGE_GEOMETRY_WITH_CABIN_SIZING = "service.geometry.fuselage.with_cabin_sizing"
SERVICE_FUSELAGE_CNBETA = "service.geometry.fuselage.cnbeta"
SERVICE_HORIZONTAL_TAIL_GEOMETRY = "service.geometry.horizontal_tail"
SERVICE_NACELLE_PYLON_GEOMETRY = "service.geometry.nacelle_and_pylon"
SERVICE_VERTICAL_TAIL_GEOMETRY = "service.geometry.vertical_tail"
SERVICE_WING_GEOMETRY = "service.geometry.wing"
SERVICE_AIRCRAFT_WETTED_AREA = "service.geometry.aircraft.wetted_area"
SERVICE_AIRCRAFT_AERODYNAMIC_CENTER = "service.geometry.aircraft.aerodynamic_center"
