"""Constants for mass submodels."""
#  This file is part of FAST-OAD_CS25
#  Copyright (C) 2022 ONERA & ISAE-SUPAERO
#  FAST is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.

SERVICE_OWE = "service.mass.owe"
SERVICE_PAYLOAD_MASS = "service.mass.payload"
SERVICE_AIRFRAME_MASS = "service.mass.airframe"
SERVICE_PROPULSION_MASS = "service.mass.propulsion"
SERVICE_SYSTEMS_MASS = "service.mass.systems"
SERVICE_FURNITURE_MASS = "service.mass.furniture"
SERVICE_CREW_MASS = "service.mass.crew"

SERVICE_GUST_LOADS = "service.gust_loads"
